# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-09 12:54
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('activity_type', models.SmallIntegerField(choices=[(0, 'favorite'), (1, 'like'), (2, 'up_vote'), (3, 'down_vote')])),
                ('active', models.BooleanField(default=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Activity',
                'verbose_name_plural': 'Activities',
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('text', models.CharField(max_length=255)),
                ('active', models.BooleanField(default=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Comment',
                'verbose_name_plural': 'Comments',
            },
        ),
        migrations.CreateModel(
            name='Rating',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=50, unique=True)),
                ('description', models.CharField(blank=True, max_length=255)),
                ('slug', models.SlugField(unique=True)),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Rating',
                'verbose_name_plural': 'Ratings',
            },
        ),
        migrations.CreateModel(
            name='Review',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('text', models.CharField(max_length=255)),
                ('active', models.BooleanField(default=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('rating', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='reviews', to='user_activities.Rating')),
            ],
            options={
                'verbose_name': 'Review',
                'verbose_name_plural': 'Reviews',
            },
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=50, unique=True)),
                ('description', models.CharField(blank=True, max_length=255)),
                ('slug', models.SlugField(unique=True)),
                ('active', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Tag',
                'verbose_name_plural': 'Tag',
            },
        ),
        migrations.AddField(
            model_name='review',
            name='tags',
            field=models.ManyToManyField(blank=True, to='user_activities.Tag'),
        ),
        migrations.AddField(
            model_name='review',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='comment',
            name='tags',
            field=models.ManyToManyField(blank=True, to='user_activities.Tag'),
        ),
        migrations.AddField(
            model_name='comment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='activity',
            unique_together=set([('user', 'activity_type', 'content_type', 'object_id')]),
        ),
    ]



# Generated by Django 2.1 on 2018-10-23 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user_activities', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='pinned',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='review',
            name='pinned',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8
from django.contrib import admin

from . import models


class ActivityAdmin(admin.ModelAdmin):
    model = models.Activity
    list_display = ('user', 'activity_type', 'content_type', 'object_id', 'active', 'created', 'modified')
    raw_id_fields = ('user', )
    search_fields = ('user__username', )
    list_filter = ('active', 'activity_type')
    save_as = True


class RatingAdmin(admin.ModelAdmin):
    model = models.Rating
    list_display = ('label', 'slug', 'active', 'created', 'modified')
    prepopulated_fields = {'slug': ('label', )}
    search_fields = ('label', )
    list_filter = ('active', )
    save_as = True


class TagAdmin(admin.ModelAdmin):
    model = models.Tag
    list_display = ('label', 'slug', 'active', 'created', 'modified')
    prepopulated_fields = {'slug': ('label', )}
    search_fields = ('label', )
    list_filter = ('active', )
    save_as = True


class CommentAdmin(admin.ModelAdmin):
    model = models.Comment
    list_display = ('user', 'content_type', 'object_id', 'active', 'created', 'modified')
    raw_id_fields = ('user', )
    search_fields = ('user__username', 'text')
    list_filter = ('active', )
    save_as = True


class ReviewAdmin(admin.ModelAdmin):
    model = models.Review
    list_display = ('user', 'content_type', 'object_id', 'rating', 'active', 'created', 'modified')
    raw_id_fields = ('user', )
    search_fields = ('user__username', 'text')
    list_filter = ('active', 'rating')
    save_as = True


admin.site.register(models.Activity, ActivityAdmin)
admin.site.register(models.Rating, RatingAdmin)
admin.site.register(models.Tag, TagAdmin)
admin.site.register(models.Comment, CommentAdmin)
admin.site.register(models.Review, ReviewAdmin)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from model_utils import Choices


ACTIVITY_TYPES = Choices(
    (0, 'favorite', _('favorite')),
    (1, 'like', _('like')),
    (2, 'up_vote', _('up_vote')),
    (3, 'down_vote', _('down_vote')),
)



# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from genomix.fields import ContentRelatedField, DisplayChoiceField, UserRelatedField
from rest_framework import serializers

from . import choices, fields, models


class ActivitySerializer(serializers.ModelSerializer):
    """Serializer for list and detail requests for User Activities."""

    user = UserRelatedField(queryset=get_user_model().objects.all())
    activity_type = DisplayChoiceField(choices=choices.ACTIVITY_TYPES)

    class Meta:
        model = models.Activity
        fields = (
            'id', 'activity_type', 'active',
            'user', 'created', 'modified',
        )


class ActivitySerializerCreateOrEdit(serializers.ModelSerializer):
    """Serializer for POST, PUT, PATCH requests for User Activities."""

    user = UserRelatedField(queryset=get_user_model().objects.all())
    activity_type = DisplayChoiceField(choices=choices.ACTIVITY_TYPES)
    content_type = ContentRelatedField(queryset=ContentType.objects.all())

    class Meta:
        model = models.Activity
        fields = (
            'id', 'activity_type', 'active',
            'content_type', 'object_id',
            'user', 'created', 'modified',
        )


class CommentSerializer(serializers.ModelSerializer):
    """Serializer for list and detail requests for User Comments."""

    user = UserRelatedField(queryset=get_user_model().objects.all())
    tags = serializers.StringRelatedField(many=True)

    class Meta:
        model = models.Comment
        fields = (
            'id', 'text', 'active', 'tags',
            'user', 'created', 'modified',
        )


class CommentSerializerCreateOrEdit(serializers.ModelSerializer):
    """Serializer for POST, PUT, PATCH requests for User Comments."""

    user = UserRelatedField(queryset=get_user_model().objects.all())
    content_type = ContentRelatedField(queryset=ContentType.objects.all())

    class Meta:
        model = models.Comment
        fields = (
            'id', 'text', 'active',
            'content_type', 'object_id',
            'user', 'created', 'modified',
        )


class ReviewSerializer(serializers.ModelSerializer):
    """Serializer for list and detail requests for User Reviews."""

    user = UserRelatedField(queryset=get_user_model().objects.all())
    tags = serializers.StringRelatedField(many=True)
    rating = fields.RatingRelatedField(queryset=models.Rating.objects.all())

    class Meta:
        model = models.Comment
        fields = (
            'id', 'text', 'active',
            'rating', 'tags',
            'user', 'created', 'modified',
        )


class ReviewSerializerCreateOrEdit(serializers.ModelSerializer):
    """Serializer for POST, PUT, PATCH requests for User Reviews."""

    user = UserRelatedField(queryset=get_user_model().objects.all())
    content_type = ContentRelatedField(queryset=ContentType.objects.all())
    rating = fields.RatingRelatedField(queryset=models.Rating.objects.all())

    class Meta:
        model = models.Review
        fields = (
            'id', 'text', 'rating', 'active',
            'content_type', 'object_id',
            'user', 'created', 'modified',
        )



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models import Count, Q


class ActivityQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('user').all()

    def aggregate_counts(self, content_type, object_id, active=True):
        counts_filter = Q(content_type=content_type)
        counts_filter &= Q(object_id=object_id)
        counts_filter &= Q(active=True)

        return self.fast()\
            .filter(counts_filter) \
            .values('activity_type') \
            .annotate(total=Count('activity_type'))

    def user_actions(self, user, content_type, object_id):
        actions_filter = Q(user__id=user.id)
        actions_filter &= Q(content_type=content_type)
        actions_filter &= Q(object_id=object_id)

        return self.fast() \
            .filter(actions_filter) \
            .values('id', 'active', 'activity_type') \
            .distinct()


class CommentQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('user').prefetch_related('tags').all()


class ReviewQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('user').select_related('rating').prefetch_related('tags').all()
